import 'package:enx_uikit_flutter/controller/enx_controller.dart';
import 'package:enx_uikit_flutter/screens/chat_ui.dart';
import 'package:enx_uikit_flutter/utilities/color.dart';
import 'package:enx_uikit_flutter/utilities/utils.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:get/get_core/src/get_main.dart';
import 'package:get/get_navigation/src/extension_navigation.dart';
import 'package:get/get_navigation/src/routes/transitions_type.dart';
import 'package:get/get_state_manager/src/rx_flutter/rx_obx_widget.dart';

import '../utilities/enx_setting.dart';
import '../widgets/more_widget.dart';
import 'custom_button.dart';

/// A UI class to style how the buttons look. Use this class to add, remove or customize the buttons in your live video calling application.
class EnxVideoButtons extends StatefulWidget {
  final EnxController enxController;
  // List of enabled buttons. Use this to remove any of the default button or change their order.
  const EnxVideoButtons({
    Key? key,
    required this.enxController,
  }) : super(key: key);

  @override
  _EnxVideoButtonsState createState() => _EnxVideoButtonsState();
}

class _EnxVideoButtonsState extends State<EnxVideoButtons> {
  List<Widget> buttonsEnabled = [];

  /// List of buttons that are added next to the default buttons. The buttons class contains a horizontal scroll view.
  final List<Widget> extraButton = [];

  @override
  Widget build(BuildContext context) {

    for (var element in buttonsEnabled) {
      print("${element.key}");

    }
    return toolbar(buttonsEnabled);
  }

  Widget groupChatButton() {
    return Obx(() => CustomMaterialButton(
          onPressed: () {
            widget.enxController.isChatIcon.value = false;
            print('asdf${widget.enxController.userList.length}');
            if (widget.enxController.participantsList.isNotEmpty ||
                widget.enxController.moderatorsList.isNotEmpty) {
              Get.to(() => ChatPage(widget.enxController, 'group'),
                  transition: Transition.leftToRight,
                  duration: const Duration(seconds: 1));
              widget.enxController.chatType.value = 'group';
            } else {
              showToastMessage('No participants available');
            }
          },
          child: Image.asset(
            "assets/image/group_chat_icon.png",
            package: 'enx_uikit_flutter',
            color: widget.enxController.isChatGroupIcon
                ? CustomColors.themeColor
                : Colors.black54,
          ),
        ));
  }

  @override
  void initState() {
    super.initState();
    Map<EnxRequiredEventsOption, Widget> buttonMap = <EnxRequiredEventsOption, Widget>{};
    var setting = EnxSetting.instance;

    if (setting.enxRequiredList.isNotEmpty) {

      for (var element in setting.enxRequiredList) {
        print('fg$element');
        if (element == EnxRequiredEventsOption.audio) {
          buttonMap.putIfAbsent(EnxRequiredEventsOption.audio, () => _muteMicButton());
        } else if (element == EnxRequiredEventsOption.video) {
          if(!widget.enxController.isSelected[2]){
            buttonMap.putIfAbsent(
                EnxRequiredEventsOption.video, () => _disableVideoButton());
          }

        } else if (element == EnxRequiredEventsOption.cameraSwitch) {
          if(!widget.enxController.isSelected[2]) {
            buttonMap.putIfAbsent(
                EnxRequiredEventsOption.cameraSwitch, () => _switchCameraButton());
          }
        } else if (element == EnxRequiredEventsOption.audioSwitch) {
          buttonMap.putIfAbsent(
              EnxRequiredEventsOption.audioSwitch, () => _switchAudioButton());
        } else if (element == EnxRequiredEventsOption.groupChat) {
          buttonMap.putIfAbsent(
              EnxRequiredEventsOption.groupChat, () => groupChatButton());
        } else if (element == EnxRequiredEventsOption.disconnect) {
          buttonMap.putIfAbsent(
              EnxRequiredEventsOption.disconnect, () => _disconnectCallButton());
        }
      }

      if(setting.containers) {
        buttonMap.putIfAbsent(
          EnxRequiredEventsOption.more, () => _moreButton());
      }


    } else {
      if(widget.enxController.isSelected[2]) {
        buttonMap = <EnxRequiredEventsOption, Widget>{
          EnxRequiredEventsOption.audio: _muteMicButton(),
          EnxRequiredEventsOption.audioSwitch: _switchAudioButton(),
          EnxRequiredEventsOption.groupChat: groupChatButton(),
          EnxRequiredEventsOption.disconnect: _disconnectCallButton(),
        };
  }else{
    buttonMap = <EnxRequiredEventsOption, Widget>{
      EnxRequiredEventsOption.audio: _muteMicButton(),
      EnxRequiredEventsOption.video: _disableVideoButton(),
      EnxRequiredEventsOption.cameraSwitch: _switchCameraButton(),
      EnxRequiredEventsOption.audioSwitch: _switchAudioButton(),
      EnxRequiredEventsOption.groupChat: groupChatButton(),
    if(!widget.enxController.isSelected[2])  EnxRequiredEventsOption.more:_moreButton(),
      EnxRequiredEventsOption.disconnect: _disconnectCallButton(),

    };
    }

    }
    for (var j = 0; j < buttonMap.length; j++) {
      buttonsEnabled.add(buttonMap.values.toList()[j]);
    }

    if (setting.extraButtons.isNotEmpty) {
      extraButton.addAll(setting.extraButtons);
    }
  }

  Widget toolbar(List<Widget> buttonList) {
    return Container(
      width: MediaQuery.of(context).orientation == Orientation.portrait
          ? MediaQuery.of(context).size.width
          : MediaQuery.of(context).size.height,
      alignment: Alignment.bottomCenter,

      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          Container(
            width: MediaQuery.of(context).orientation == Orientation.portrait
                ? MediaQuery.of(context).size.width
                : MediaQuery.of(context).size.height,
            padding: EdgeInsets.only(
                bottom: MediaQuery.of(context).orientation == Orientation.portrait
                    ? MediaQuery.of(context).size.width/20
                    : MediaQuery.of(context).size.height/20,
                top: 5,
                right: 3),
            decoration: BoxDecoration(
                color: Colors.white),
            child: Center(
              child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceEvenly,

                  children: [
                for (var i = 0; i < buttonList.length; i++) buttonList[i],
                if (extraButton.isNotEmpty)
                  for (var i = 0; i < extraButton.length; i++) extraButton[i],
              ]),
            ),
            // ),
          ),
        ],
      ),
    );
  }

  Widget _disableVideoButton() {
    return Obx(() => CustomMaterialButton(
          onPressed: () => widget.enxController.toggleMuteVideo(),
          child: Icon(
            widget.enxController.isVideoMute ||
                    widget.enxController.isHardMuteVideo
                ? Icons.videocam_off_outlined
                : Icons.videocam_outlined,
            color: widget.enxController.isVideoMute ||
                    widget.enxController.isHardMuteVideo
                ? CustomColors.themeColor
                : Colors.grey,
            size: 25.0,
          ),
          elevation: 2.0,
          padding: const EdgeInsets.all(12.0),
        ));
  }

  Widget _disconnectCallButton() {
    return CustomMaterialButton(
      onPressed: () => _onCallEnd(context),
      child: const Icon(Icons.call_rounded, color: Colors.white, size: 20),
      shape: const CircleBorder(),
      elevation: 2.0,
      fillColor: CustomColors.themeColor,
    );
  }

  Widget _moreButton() {
    return CustomMaterialButton(
      onPressed: () => {
        showMoreDialog(context,widget.enxController)
      },
      child: const Icon(Icons.more_horiz, color: Colors.grey, size: 30),

    );
  }

  Widget _muteMicButton() {
    return Obx(() => CustomMaterialButton(
          onPressed: () => widget.enxController.toggleMuteAudio(),
          child: Icon(
            (widget.enxController.isAudioMute ||
                    widget.enxController.isHardMuteRoomAudio)
                ? Icons.mic_off_outlined
                : Icons.mic_none_outlined,
            color: (widget.enxController.isAudioMute ||
                    widget.enxController.isHardMuteRoomAudio)
                ? CustomColors.themeColor
                : Colors.grey,
            size: 30.0.w,
          ),
        ));
  }

  Widget _muteRoomButton() {
    return Visibility(
      visible: widget.enxController.isRole ? true : false,
      child: CustomMaterialButton(
        onPressed: () => widget.enxController.muteRoom(),
        child: Stack(children: [
          Icon(
            Icons.perm_identity_outlined,
            color: widget.enxController.isHardMuteRoom
                ? CustomColors.themeColor
                : Colors.grey,
            size: 36.0,
          ),
          Positioned(
            left: 12,
            child: Icon(
              widget.enxController.isHardMuteRoom
                  ? Icons.mic_off_outlined
                  : Icons.mic_none_outlined,
              color: widget.enxController.isHardMuteRoom
                  ? CustomColors.themeColor
                  : Colors.grey,
              size: 30.0,
            ),
          )
        ]),
      ),
    );
  }

  /// Default functionality of disconnect button is such that it pops the view and navigates the user to the previous screen.
  void _onCallEnd(BuildContext context) {
    widget.enxController.endCall('');
    widget.enxController.dispose();
    Get.back();
  }

  Widget _recordingRoomButton() {
    return Visibility(
      visible: widget.enxController.isRole ? true : false,
      child: CustomMaterialButton(
        onPressed: () => widget.enxController.startRecording(),
        child: Icon(
          Icons.radio_button_checked,
          color: widget.enxController.isRecord
              ? CustomColors.themeColor
              : Colors.grey,
          size: 25.0,
        ),
      ),
    );
  }

  Widget _switchAudioButton() {
    return Obx(() => CustomMaterialButton(
          onPressed: () =>
              widget.enxController.switchMedia(widget.enxController),
          child: Icon(
            Icons.volume_up_outlined,
            color: widget.enxController.selectedMedia ==
                        'EARPIECE'.toUpperCase() ||
                    widget.enxController.selectedMedia == 'Earpiece'
                ? Colors.grey
                : CustomColors.themeColor,
            size: 27.0,
          ),
        ));
  }

  Widget _switchCameraButton() {
    return Obx(() => CustomMaterialButton(
          onPressed: () {
            if (!widget.enxController.isVideoMute) {
              widget.enxController.switchCamera();
            }
          },
          child: Icon(
            Icons.flip_camera_ios_outlined,
            color: widget.enxController.isBackCamera
                ? CustomColors.themeColor
                : Colors.grey,
            size: 27.0,
          ),
        ));
  }

  Widget _switchViewButton() {
    return CustomMaterialButton(
      onPressed: () => widget.enxController.switchAtView(),
      child: Icon(
        Icons.window_outlined,
        color: widget.enxController.isGridView
            ? Colors.grey
            : CustomColors.themeColor,
        size: 25.0,
      ),
    );
  }
}
