import 'dart:convert';

import 'package:enx_uikit_flutter/utilities/request.dart';
import 'package:enx_uikit_flutter/utilities/utils.dart';
import 'package:flutter/material.dart';
import 'package:get/get_core/src/get_main.dart';
import 'package:get/get_navigation/src/extension_navigation.dart';

class EnxSetting with ChangeNotifier {
  static final EnxSetting _instance = EnxSetting._privateConstructor();

  static EnxSetting get instance => _instance;

  Set<BottomOption> bottomList = <BottomOption>{};
  bool containers=false;
  Set<EnxRequiredEventsOption> enxRequiredList=<EnxRequiredEventsOption>{};
  Set<TopOption> topList = <TopOption>{};
  Set<MenuOption> menuList = <MenuOption>{};
  Set<ParticipantListOption> participantListOption=<ParticipantListOption>{};
  final Set<Widget> extraButtons = {};
  Map<dynamic, dynamic> visualBuilder = {};

  Map<dynamic, dynamic> optionsData = {};
  bool isVideo = false;
  bool isAudio = false;
  bool isCameraSwitch = true;
  bool isAudioSwitch = true;
  bool isChat = false;
  bool isDisconnect = true;
  String groupName = "", getClientId = '';

  bool isStart = false;

  EnxSetting._privateConstructor();

  createBottomOption(BottomOption btn) {
    bottomList.add(btn);
  }

  createExtraButton(List<Widget> btn) {
    if (extraButtons.isEmpty) {
      extraButtons.addAll(btn);
    }
  }

  createMenuOptions(MenuOption btn) {
    menuList.add(btn);
  }

  createTopOption(TopOption btn) {
    topList.add(btn);
  }

  configureRequiredEventList(Set<EnxRequiredEventsOption> btn){
    enxRequiredList.addAll(btn);

    var tagList=<String>{};

    if(enxRequiredList.isNotEmpty){
      for (var element in enxRequiredList) {

        tagList.add(element.name);

      }

      for(var item in MenuOption.values){
        if (tagList.contains(item.name)) {
          containers = true;
          break;
        }
      }
    }


  }


  configureParticipantList(ParticipantListOption btn){
    participantListOption.add(btn);

  }
  Set<EnxRequiredEventsOption> getEnxRequiredEventList() {
    return enxRequiredList;
  }

  Set<ParticipantListOption> getParticipantOptionList() {
    return participantListOption;
  }

    Set<BottomOption> getBottomOptionList() {
    return bottomList;
  }

  void getEmbedSetting({required String embedUrl}) async {
    Requests request = Requests(url: embedUrl + '/config/index.json');
    await request.get().then((value) {
      //print(value.body);
      final parseJson = jsonDecode(value.body);
      optionsData = parseJson['hosted_apps']['vc']['options'];
      print("visualBUilder$optionsData");
      //  final userdata = Map<String, dynamic>.from(parseJson['hosted_apps']['vc']['options']);
      if (optionsData.isNotEmpty) {
        isAudio = optionsData['audio_mute'];
        if (isAudio) {
          bottomList.add(BottomOption.audio);
        }

        isVideo = optionsData['video_mute'];

        if (isVideo) {
          bottomList.add(BottomOption.video);
        }

        if (isCameraSwitch) {
          bottomList.add(BottomOption.cameraSwitch);
        }
        if (isAudioSwitch) {
          bottomList.add(BottomOption.audioSwitch);
        }

        isChat = optionsData['group_chat'];
        if (isChat) {
          bottomList.add(BottomOption.groupChat);
        }
        if (isDisconnect) {
          bottomList.add(BottomOption.disconnect);
        }

        if (optionsData['mute_all'] == true) {
          menuList.add(MenuOption.muteRoom);
        }
        if (optionsData['recording'] == true) {
          menuList.add(MenuOption.recording);
        }

        if (optionsData['grid_mode_switch'] == true) {
          menuList.add(MenuOption.switchAT);
        }
      }

      //  onReceivedSetting(jsonDecode(value.body));
    }).catchError((onError) {});
  }



  Set<Widget> getExtraButtons() {
    return extraButtons;
  }

  Set<MenuOption> getMenuOptionList() {
    return menuList;
  }

  getScreenShare(bool isScreen) {
    isStart = isScreen;
  }

  getSelfClientId(String id) {
    if (id.isNotEmpty) {
      getClientId = id;
    } else {
      showToastMessage(roomNotConnected);
    }
  }

  Set<TopOption> getTopOptionList() {
    return topList;
  }

  setGroupName(String grpName) {
    return groupName = grpName;
  }


  startShare(bool value) {
    isStart = value;
    notifyListeners();
  }
}

enum BottomOption {
  audio,
  video,
  cameraSwitch,
  audioSwitch,
  groupChat,
  disconnect,
  none
}

enum MenuOption { switchAT, muteRoom, recording,requestFloor, annotationShare }
enum TopOption {  menu, requestFloor, none }

enum EnxRequiredEventsOption { audio,
  video,
  cameraSwitch,
  audioSwitch,
  groupChat,
  disconnect,
  more,
  switchAT, muteRoom, recording, annotationShare,requestFloor, none

}

enum ParticipantListOption { audio, video, chat, disconnect }
